def create_gdt_null_descriptor_code(boot, log)
  boot.puts "gdt_start: "
  boot.puts "gdt_null_desc: "
  boot.puts "   dd 0"
  boot.puts "   dd 0"
end

def create_gdt_code_descriptor_code(boot, log, dpl, limit, base, conforming)
  boot.puts "gdt_sys_code_desc: "
  boot.puts "   dw 0x" + (limit & 0xFFFF).to_s(16)
  boot.puts "   dw 0x" + (base & 0xFFFF).to_s(16)
  boot.puts "   db 0x" + ((base & 0xFF0000) >> 16).to_s(16)
  boot.puts "   db 0x" + (0x9A + ((dpl << 5) + (conforming << 2))).to_s(16)
  boot.puts "   db 0x" + (0xC0 + ((limit & 0xF0000) >> 16)).to_s(16)
  boot.puts "   db 0x" + ((base & 0xFF000000) >> 24).to_s(16)
end

def create_gdt_data_descriptor_code(boot, log, dpl, limit, base, writable)
  boot.puts "gdt_sys_data_desc: "
  boot.puts "   dw 0x" + (limit & 0xFFFF).to_s(16)
  boot.puts "   dw 0x" + (base & 0xFFFF).to_s(16)
  boot.puts "   db 0x" + ((base & 0xFF0000) >> 16).to_s(16)
  boot.puts "   db 0x" + (0x90 + ((dpl << 5) + (writable << 1))).to_s(16)
  boot.puts "   db 0x" + (0xC0 + ((limit & 0xF0000) >> 16)).to_s(16)
  boot.puts "   db 0x" + ((base & 0xFF000000) >> 24).to_s(16)
end

def create_gdt_desc_table_code(boot, log)
  boot.puts "gdt_end: "
  boot.puts "gdt_desc: "                     
  boot.puts "   dw gdt_end - gdt_start - 1"   
  boot.puts "   dd gdt_start"
  boot.puts "\n" 
end

def get_gdt_code_desc_info(boot, log)
  print "\nGDT Code Descriptor Info: \n"
  print "DPL [0-3]: "
  code_desc_dpl = gets.chomp
  print "Limit [0-0xFFFFF]: "
  code_desc_limit = gets.chomp
  print "Base [0-0xFFFFFFFF]: "
  code_desc_base = gets.chomp
  print "Conforming [0-1]: "
  code_desc_c = gets.chomp
  
  if code_desc_dpl[0..1] == "0x"
    code_desc_dpl = code_desc_dpl.hex
  else
    code_desc_dpl = code_desc_dpl.to_i
  end
  if code_desc_dpl > 3
    log.puts "code_desc_dpl = 0x" + code_desc_dpl.to_s
    log.puts "code_desc_dpl is not formatted correctly. (value exceeds boundaries)"
  end

  if code_desc_limit[0..1] == "0x"
    code_desc_limit = code_desc_limit.hex
  else
    code_desc_limit = code_desc_limit.to_i
  end
  if code_desc_limit > 0xFFFFF
    log.puts "code_desc_limit = 0x" + code_desc_limit.to_s
    log.puts "code_desc_limit is not formatted correctly. (value exceeds boundaries)"
  end

  if code_desc_base[0..1] == "0x"
    code_desc_base = code_desc_base.hex
  else
    code_desc_base = code_desc_base.to_i
  end
  if code_desc_base > 0xFFFFFFFF
    log.puts "code_desc_base = 0x" + code_desc_base.to_s
    log.puts "code_desc_base is not formatted correctly. (value exceeds boundaries)"
  end

  if code_desc_c[0..1] == "0x"
    code_desc_c = code_desc_c.hex
  else
    code_desc_c = code_desc_c.to_i
  end
  if code_desc_c > 1
    log.puts "code_desc_c = 0x" + code_desc_c.to_s
    log.puts "code_desc_c is not formatted correctly. (value exceeds boundaries)"
  end

  log.puts "GDT Code Descriptor: 0x" + code_desc_dpl.to_s(16) + " | 0x" + code_desc_limit.to_s(16) + 
                             " | 0x" + code_desc_base.to_s(16) + " | 0x" + code_desc_c.to_s(16)
  
  create_gdt_code_descriptor_code(boot, log, code_desc_dpl, code_desc_limit, code_desc_base, code_desc_c)
end

def get_gdt_data_desc_info(boot, log)
  print "\nGDT Data Descriptor Info: \n"
  print "DPL [0-3]: "
  data_desc_dpl = gets.chomp
  print "Limit [0-0xFFFFF]: "
  data_desc_limit = gets.chomp
  print "Base [0-0xFFFFFFFF]: "
  data_desc_base = gets.chomp
  print "Writable [0-1]: "
  data_desc_w = gets.chomp
  
  if data_desc_dpl[0..1] == "0x"
    data_desc_dpl = data_desc_dpl.hex
  else
    data_desc_dpl = data_desc_dpl.to_i
  end
  if data_desc_dpl > 3
    log.puts "data_desc_dpl = 0x" + data_desc_dpl.to_s
    log.puts "data_desc_dpl is not formatted correctly. (value exceeds boundaries)"
  end

  if data_desc_limit[0..1] == "0x"
    data_desc_limit = data_desc_limit.hex
  else
    data_desc_limit = data_desc_limit.to_i
  end
  if data_desc_limit > 0xFFFFF
    log.puts "data_desc_limit = 0x" + data_desc_limit.to_s
    log.puts "data_desc_limit is not formatted correctly. (value exceeds boundaries)"
  end

  if data_desc_base[0..1] == "0x"
    data_desc_base = data_desc_base.hex
  else
    data_desc_base = data_desc_base.to_i
  end
  if data_desc_base > 0xFFFFFFFF
    log.puts "data_desc_base = 0x" + data_desc_base.to_s
    log.puts "data_desc_base is not formatted correctly. (value exceeds boundaries)"
  end

  if data_desc_w[0..1] == "0x"
    data_desc_w = data_desc_w.hex
  else
    data_desc_w = data_desc_w.to_i
  end
  if data_desc_w > 1
    log.puts "data_desc_w = 0x" + data_desc_w.to_s
    log.puts "data_desc_w is not formatted correctly. (value exceeds boundaries)"
  end

  log.puts "GDT Data Descriptor: 0x" + data_desc_dpl.to_s(16) + " | 0x" + data_desc_limit.to_s(16) + 
                             " | 0x" + data_desc_base.to_s(16) + " | 0x" + data_desc_w.to_s(16)
  
  create_gdt_data_descriptor_code(boot, log, data_desc_dpl, data_desc_limit, data_desc_base, data_desc_w)
end

def create_gdt_descriptor_code(boot, log)
  create_gdt_null_descriptor_code(boot, log)
  get_gdt_code_desc_info(boot, log)
  get_gdt_data_desc_info(boot, log)
  create_gdt_desc_table_code(boot, log)
end
  

