def setup_page_tables_code(boot, log, page_table_base, phys_base, page_tables)
  boot.puts "setup_page_tables: "
  boot.puts "   mov eax, 0x" + (page_tables * 0x1000).to_s(16)
  boot.puts "   mov ebx, 0x" + page_table_base.to_s(16)
  boot.puts "   mov ecx, 0x00000000"
  boot.puts "   mov edx, 0x" + phys_base.to_s(16)
  boot.puts "setup_page_tables_loop: "
  boot.puts "   push edx"
  boot.puts "   or edx, 3"
  boot.puts "   mov [ebx + ecx], edx"
  boot.puts "   pop edx"
  boot.puts "   add edx, 0x1000"
  boot.puts "   add ecx, 4"
  boot.puts "   cmp ecx, eax"
  boot.puts "   jb setup_page_tables_loop"
  boot.puts "\n"
end

def setup_page_directories_code(boot, log, page_directory_base, page_table_base, page_tables)
  boot.puts "setup_page_directories: "
  boot.puts "   mov eax, 0x" + (page_tables * 4).to_s(16)
  boot.puts "   mov ebx, 0x" + page_directory_base.to_s(16)
  boot.puts "   mov ecx, 0x00000000"
  boot.puts "   mov edx, 0x" + page_table_base.to_s(16)
  boot.puts "setup_page_directories_loop: "
  boot.puts "   push edx"
  boot.puts "   or edx, 3"
  boot.puts "   mov [ebx + ecx], edx"
  boot.puts "   pop edx"
  boot.puts "   add edx, 0x1000"
  boot.puts "   add ecx, 4"
  boot.puts "   cmp ecx, eax"
  boot.puts "   jb setup_page_directories_loop"
  boot.puts "setup_unused_page_directories_loop: "
  boot.puts "   mov [ebx + ecx], dword 0x00000002"
  boot.puts "   add ecx, 4"
  boot.puts "   cmp ecx, 0x1000"
  boot.puts "   jb setup_unused_page_directories_loop"
  boot.puts "\n"
end

def enable_paging_code(boot, log, page_directory_base)
  boot.puts "enable_paging: "
  boot.puts "   mov eax, 0x" + page_directory_base.to_s(16)
  boot.puts "   mov cr3, eax"
  boot.puts "   mov eax, cr0"
  boot.puts "   or eax, 0x80000000"
  boot.puts "   mov cr0, eax"
  boot.puts "\n"
end

def get_more_paging_info(boot, log)
  puts "\nPaging Info: "
  print "Physical Base Address [0-0xFFFFFFFF]: "
  phys_base = gets.chomp.hex
  print "Page Tables Start Address [0-0xFFFFFFFF]: "
  page_table_base = gets.chomp.hex
  print "Number of Page Tables [0-0x1000]: "
  page_tables = gets.chomp.hex
  page_directory_base = (page_table_base + (0x1000 * page_tables))
  log.puts "Page Tables: 0x" + phys_base.to_s(16) + " | 0x" + page_table_base.to_s(16) + 
           " | 0x" + page_tables.to_s(16) + " | 0x" + page_directory_base.to_s(16)
           
  setup_page_tables_code(boot, log, page_table_base, phys_base, page_tables)
  setup_page_directories_code(boot, log, page_directory_base, page_table_base, page_tables)
  enable_paging_code(boot, log, page_directory_base)
end

def get_paging_info(boot, log)
  loop {
    puts "\nWould you like to enable paging? "
    print "[Y/N]: "
    resp = gets.chomp.upcase
    if resp.length == 1
      if resp == "N" 
        log.puts "user has requested that paging be disabled"
        break
      end
      if resp == "Y"
        log.puts "user has requested that paging be enabled"
        get_more_paging_info(boot, log)
        break
      end
    end
    puts "Invalid Response."
    log.puts "paging - invalid response ('" + resp + "')"
  }
end










